#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout(std430 ) buffer BInOutCounts
{
	uint InOutCounts[];
} InOutCounts;

layout( std140) uniform UInfo
{
	uint CurrentBit;
	uint GroupCount;
} Info;

shared uint groupCounts[256];


void ComputeMain( in SComputeInput  cInput  )
{
	const uint localThreadID = cInput.LocalThreadID.x;

	const uint groupCount =  Info.GroupCount ;

	uint lastTotal = 0;

	for (uint offseti = 0; offseti < groupCount; offseti +=  128  * 2)
	{
		uvec2 counts = uvec2(0, 0);
		if ((offseti + localThreadID * 2) < groupCount)
			counts =  uvec2( InOutCounts.InOutCounts [ (offseti + localThreadID * 2) ], InOutCounts.InOutCounts [( (offseti + localThreadID * 2) ) + 1]) ;

		groupCounts [localThreadID * 2 + 0] = counts.x;
		groupCounts [localThreadID * 2 + 1] = counts.y;

		uint stride = 1;
		for (uint d =  128 ; d > 0; d /= 2)
		{
			groupMemoryBarrier(); barrier(); ;
			if (localThreadID < d)
			{
				const uint ai = (localThreadID * 2 + 1) * stride - 1;
				const uint bi = (localThreadID * 2 + 2) * stride - 1;
				groupCounts [bi] +=  groupCounts [ai];
			}
			stride *= 2;
		}

		groupMemoryBarrier(); barrier(); ;
		const uint total =  groupCounts [ 128  * 2 - 1];

		for (uint d = 1; d <  128  * 2; d *= 2)
		{
			stride /= 2;
			groupMemoryBarrier(); barrier(); ;
			if (localThreadID > 0 && localThreadID < d)
			{
				const uint ai = (localThreadID * 2 + 0) * stride - 1;
				const uint bi = (localThreadID * 2 + 1) * stride - 1;
				groupCounts [bi] +=  groupCounts [ai];
			}
		}

		groupMemoryBarrier(); barrier(); ;

		const uvec2 outOffsets = uvec2( groupCounts [localThreadID * 2 + 0],  groupCounts [localThreadID * 2 + 1]) + lastTotal;
		InOutCounts.InOutCounts [ (offseti + localThreadID * 2) ] = outOffsets.x; InOutCounts.InOutCounts [( (offseti + localThreadID * 2) ) + 1] = outOffsets.y ;

		lastTotal += total;
	}
}

layout(local_size_x = 128, local_size_y = 1, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
